//////////////////////////////////////////////////////////
/********************************************************/
/*                                                      */
/*  Fachhochschule Jena                                 */
/*  Mikroprozessortechnik                               */
/*  Michael Goldbuch; Jrgen Dffinger                  */
/*                                                      */
/*  USART.asm   Version 1.0.0   2011-05-17              */
/*                                                      */
/*  Die USART.asm enthlt alle Funktionen zur           */
/*  Kommunikation ber USART                            */
/*                                                      */
/********************************************************/
//////////////////////////////////////////////////////////



/***************   USART_INIT   ***************/


USART_INIT:

	push R16
	push R17


	; Set baud rate (2400 BAUD)

	clr R16

	ldi R17,0xD0

	out UBRRH, R16

	out UBRRL, R17



	; Enable receiver and transmitter and the interupts

	ldi R16, (1<<TXEN)|(1<<TXCIE)

	out UCSRB,R16

	

	; Set frame format: 8data, 2stop bit

	ldi R16, (1<<URSEL)|(1<<USBS)|(3<<UCSZ0)

	out UCSRC,R16


	pop R17
	pop R16

ret



/***************   Transmit   ***************/

.macro Transmit

	push  R16
	mov   R16,@0
	rcall TX
	pop   R16

.endm

TX:

	//	Warten bis UDR frei ist	
	sbis UCSRA,UDRE
	rjmp TX

	//	zu sendendes Zeichen an UDR bergeben
	out  UDR,R16
	
ret



/***************   TX_BIN2ASCII   ***************/

.macro TX_BIN2ASCII
	
	push R16
	push R17
	push R30
	push R31

	mov  R16,@0
	mov  R17,@1

	rcall BIN2ASCII
	rcall Transmit_ASCII

	pop  R31
	pop  R30
	pop  R17
	pop  R16

.endm



/***************   BIN2ASCII   ***************/

BIN2ASCII:

	//	Register sichern
	push R0
	push R1
	push R2
	push R18
	push R20

	//	Initialisierung
	clr  R31			//	z-Pointers auf $60 setzen
	ldi  R30,$60
	ldi  R18,0x0A		//	Divisor auf 10 setzen
	ldi  R20,'0'		//	ASCII Wert fr null in R20 speichern

	//	Division durchfhren
	DIV R16,R17,R18

BIN2ASCII1:
	
	add  R2,R20			//	Rest der Division mit ASCII-Wert der Null addieren (Ergebnis ASCII-Wert des Zahlenwertes)
	st	 z+,R2			//	ASCII - Zeichen speichern und z-pointer inkrementieren
	DIV  R0,R1,R18   
	tst  R0				//	prfen ob hherwertiges Byte gleich null ist 
	brne BIN2ASCII1		//	wenn nicht dann weiter mit der Umwandlung
	tst  R1				//	niederwertiges Byte prfen ob es null ist
	brne BIN2ASCII1		//	wenn nicht dann weiter mit Umwandlung
	
	//	wenn alle Berechnungen durchgefhrt wurden, Endadresse der gespeicherten ASCII-Zeichen sichern
	add  R2,R20			//	Rest der Division mit ASCII-Wert der Null addieren (Ergebnis ASCII-Wert des Zahlenwertes)
	st	 z+,R2			//	ASCII - Zeichen speichern und z-pointer inkrementieren

	//	Resgister wiederherstellen
	pop  R20
	pop  R18
	pop  R2
	pop  R1
	pop  R0

ret



/***************   Transmit_ASCII   ***************/

Transmit_ASCII:
	
	//	Z-Pointer dekrementieren
	dec  R30
	brne Transmit_ASCII2
	ldi  R30,0xFF
	dec  R31

Transmit_ASCII2:

	//	ASCII-Zeichen laden und senden
	ld    R16,z
	Transmit R16

	//	Prfen ob Adresse $60 erreicht wurde. Wenn nicht nchstes Zeichen senden
	cpi  R31,0x00
	brne Transmit_ASCII
	cpi  R30,$60
	brne Transmit_ASCII

	// Leerzeichen senden
	ldi  R16,0x20
	Transmit R16

ret

